@@ -24,6 +24,7 @@
 #include <string.h>
 #include <syslog.h>
 #include <stdlib.h>
+#include <limits.h>
 
 #include <X11/extensions/Xinerama.h>
 
@@ -31,34 +32,12 @@
 #include "vdagent-x11.h"
 #include "vdagent-x11-priv.h"
 
-static int caught_error;
-static int (*old_error_handler)(Display *, XErrorEvent *);
-
 static int error_handler(Display *display, XErrorEvent *error)
 {
-    caught_error = 1;
+    vdagent_x11_caught_error = 1;
     return 0;
 }
 
-static void arm_error_handler(struct vdagent_x11 *x11)
-{
-    caught_error = 0;
-    XSync(x11->display, False);
-    old_error_handler = XSetErrorHandler(error_handler);
-}
-
-static int check_error_handler(struct vdagent_x11 *x11)
-{
-    int error;
-
-    XSync(x11->display, False);
-    XSetErrorHandler(old_error_handler);
-    error = caught_error;
-    caught_error = 0;
-
-    return error;
-}
-
 static XRRModeInfo *mode_from_id(struct vdagent_x11 *x11, int id)
 {
     int i;
@@ -115,9 +94,9 @@ static void update_randr_res(struct vdagent_x11 *x11, int poll)
 
     free_randr_resources(x11);
     if (poll)
-        x11->randr.res = XRRGetScreenResources(x11->display, x11->root_window);
+        x11->randr.res = XRRGetScreenResources(x11->display, x11->root_window[0]);
     else
-        x11->randr.res = XRRGetScreenResourcesCurrent(x11->display, x11->root_window);
+        x11->randr.res = XRRGetScreenResourcesCurrent(x11->display, x11->root_window[0]);
     x11->randr.outputs = malloc(x11->randr.res->noutput * sizeof(*x11->randr.outputs));
     x11->randr.crtcs = malloc(x11->randr.res->ncrtc * sizeof(*x11->randr.crtcs));
     for (i = 0 ; i < x11->randr.res->noutput; ++i) {
@@ -131,7 +110,7 @@ static void update_randr_res(struct vdagent_x11 *x11, int poll)
                                              x11->randr.res->crtcs[i]);
     }
     /* XXX is this dynamic? should it be cached? */
-    if (XRRGetScreenSizeRange(x11->display, x11->root_window,
+    if (XRRGetScreenSizeRange(x11->display, x11->root_window[0],
                               &x11->randr.min_width,
                               &x11->randr.min_height,
                               &x11->randr.max_width,
@@ -144,6 +123,12 @@ void vdagent_x11_randr_init(struct vdagent_x11 *x11)
 {
     int i;
 
+    if (x11->screen_count > 1) {
+        syslog(LOG_WARNING, "X-server has more then 1 screen, "
+               "disabling client -> guest resolution syncing");
+        return;
+    }
+
     if (XRRQueryExtension(x11->display, &i, &i)) {
         XRRQueryVersion(x11->display, &x11->xrandr_major, &x11->xrandr_minor);
         if (x11->xrandr_major == 1 && x11->xrandr_minor >= 3)
@@ -236,12 +221,12 @@ static void delete_mode(struct vdagent_x11 *x11, int output_index,
             break;
     }
     if (m < x11->randr.res->nmode) {
-        arm_error_handler(x11);
+        vdagent_x11_set_error_handler(x11, error_handler);
         XRRDeleteOutputMode (x11->display, x11->randr.res->outputs[output_index],
                              mode->id);
         XRRDestroyMode (x11->display, mode->id);
 	// ignore race error, if mode is created by others
-	check_error_handler(x11);
+	vdagent_x11_restore_error_handler(x11);
     }
 
     /* silly to update everytime for more then one monitor */
@@ -336,10 +321,10 @@ static XRRModeInfo *create_new_mode(struct vdagent_x11 *x11, int output_index,
     set_reduced_cvt_mode(&mode, width, height);
     mode.modeFlags = 0;
     mode.id = 0;
-    arm_error_handler(x11);
-    XRRCreateMode (x11->display, x11->root_window, &mode);
+    vdagent_x11_set_error_handler(x11, error_handler);
+    XRRCreateMode (x11->display, x11->root_window[0], &mode);
     // ignore race error, if mode is created by others
-    check_error_handler(x11);
+    vdagent_x11_restore_error_handler(x11);
 
     /* silly to update everytime for more then one monitor */
     update_randr_res(x11, 0);
@@ -429,7 +414,7 @@ static int set_screen_to_best_size(struct vdagent_x11 *x11, int width, int heigh
     XRRScreenConfiguration *config;
     Rotation rotation;
 
-    sizes = XRRSizes(x11->display, x11->screen, &num_sizes);
+    sizes = XRRSizes(x11->display, 0, &num_sizes);
     if (!sizes || !num_sizes) {
         syslog(LOG_ERR, "XRRSizes failed");
         return 0;
@@ -457,13 +442,13 @@ static int set_screen_to_best_size(struct vdagent_x11 *x11, int width, int heigh
         return 0;
     }
 
-    config = XRRGetScreenInfo(x11->display, x11->root_window);
+    config = XRRGetScreenInfo(x11->display, x11->root_window[0]);
     if(!config) {
         syslog(LOG_ERR, "get screen info failed");
         return 0;
     }
     XRRConfigCurrentConfiguration(config, &rotation);
-    XRRSetScreenConfig(x11->display, config, x11->root_window, best,
+    XRRSetScreenConfig(x11->display, config, x11->root_window[0], best,
                        rotation, CurrentTime);
     XRRFreeScreenConfigInfo(config);
 
@@ -476,20 +461,20 @@ static int set_screen_to_best_size(struct vdagent_x11 *x11, int width, int heigh
 }
 
 void vdagent_x11_randr_handle_root_size_change(struct vdagent_x11 *x11,
-                                               int width, int height)
+    int screen, int width, int height)
 {
-    if (width == x11->width && height == x11->height) {
+    if (width == x11->width[screen] && height == x11->height[screen]) {
         return;
     }
 
     if (x11->debug)
-        syslog(LOG_DEBUG, "Root size changed to %dx%d send %d",
-               width, height, !x11->dont_send_guest_xorg_res);
+        syslog(LOG_DEBUG, "Root size of screen %d changed to %dx%d send %d",
+              screen,  width, height, !x11->dont_send_guest_xorg_res);
 
-    x11->width  = width;
-    x11->height = height;
+    x11->width[screen]  = width;
+    x11->height[screen] = height;
     if (!x11->dont_send_guest_xorg_res) {
-        vdagent_x11_send_daemon_guest_xorg_res(x11);
+        vdagent_x11_send_daemon_guest_xorg_res(x11, 1);
     }
 }
 
@@ -537,6 +522,11 @@ static void constrain_to_screen(struct vdagent_x11 *x11, int *w, int *h)
     }
 }
 
+static int monitor_enabled(VDAgentMonConfig *mon)
+{
+    return mon->width != 0 && mon->height != 0;
+}
+
 /*
  * The agent config doesn't contain a primary size, just the monitors, but
  * we need to total size as well, to make sure we have enough memory and
@@ -553,20 +543,12 @@ static void zero_base_monitors(struct vdagent_x11 *x11,
                                VDAgentMonitorsConfig *mon_config,
                                int *width, int *height)
 {
-    int i = 0;
-    int min_x, min_y, max_x, max_y;
+    int i, min_x = INT_MAX, min_y = INT_MAX, max_x = INT_MIN, max_y = INT_MIN;
     int *mon_height, *mon_width;
 
-    mon_config->monitors[0].x &= ~7;
-    mon_config->monitors[0].width &= ~7;
-    mon_width = (int *)&mon_config->monitors[i].width;
-    mon_height = (int *)&mon_config->monitors[i].height;
-    constrain_to_screen(x11, mon_width, mon_height);
-    min_x = mon_config->monitors[0].x;
-    min_y = mon_config->monitors[0].y;
-    max_x = mon_config->monitors[0].width + min_x;
-    max_y = mon_config->monitors[0].height + min_y;
-    for (++i ; i < mon_config->num_of_monitors; ++i) {
+    for (i = 0; i < mon_config->num_of_monitors; i++) {
+        if (!monitor_enabled(&mon_config->monitors[i]))
+            continue;
         mon_config->monitors[i].x &= ~7;
         mon_config->monitors[i].width &= ~7;
         mon_width = (int *)&mon_config->monitors[i].width;
@@ -581,6 +563,8 @@ static void zero_base_monitors(struct vdagent_x11 *x11,
         syslog(LOG_ERR, "%s: agent config %d,%d rooted, adjusting to 0,0.",
                __FUNCTION__, min_x, min_y);
         for (i = 0 ; i < mon_config->num_of_monitors; ++i) {
+            if (!monitor_enabled(&mon_config->monitors[i]))
+                continue;
             mon_config->monitors[i].x -= min_x;
             mon_config->monitors[i].y -= min_y;
         }
@@ -591,73 +575,87 @@ static void zero_base_monitors(struct vdagent_x11 *x11,
     *height = max_y;
 }
 
-static int same_monitor_configs(struct vdagent_x11 *x11,
-                                VDAgentMonitorsConfig *mon,
-                                int primary_w, int primary_h)
+static int enabled_monitors(VDAgentMonitorsConfig *mon)
+{
+    int i, enabled = 0;
+
+    for (i = 0; i < mon->num_of_monitors; i++) {
+        if (monitor_enabled(&mon->monitors[i]))
+            enabled++;
+    }
+    return enabled;
+}
+
+static int same_monitor_configs(VDAgentMonitorsConfig *conf1,
+                                VDAgentMonitorsConfig *conf2)
 {
     int i;
-    XRRModeInfo *mode;
-    XRRCrtcInfo *crtc;
-    VDAgentMonConfig *client_mode;
-    XRRScreenResources *res;
 
-    update_randr_res(x11, 0);
-    res = x11->randr.res;
-
-    if (res->noutput > res->ncrtc) {
-        syslog(LOG_ERR, "error: unexpected noutput > ncrtc in driver");
-        return 0;
-    }
-
-    if (mon->num_of_monitors > res->noutput) {
-        for (i = res->noutput; i < mon->num_of_monitors; i++) {
-            if (mon->monitors[i].width || mon->monitors[i].height) {
-                syslog(LOG_WARNING,
-                       "warning: unexpected client request: #mon %d > driver output %d",
-                       mon->num_of_monitors, res->noutput);
-                break;
-            }
-        }
-        mon->num_of_monitors = res->noutput;
-    }
-
-    if (x11->width != primary_w || x11->height != primary_h)
+    if (conf1 == NULL || conf2 == NULL ||
+            conf1->num_of_monitors != conf2->num_of_monitors)
         return 0;
 
-    if (x11->randr.num_monitors != mon->num_of_monitors)
-        return 0;
-
-    for (i = 0 ; i < mon->num_of_monitors; ++i) {
-        if (x11->randr.outputs[i]->ncrtc == 0) {
-            return 0;
-        }
-        if (x11->randr.outputs[i]->ncrtc != 1) {
-            syslog(LOG_ERR, "error: unexpected driver config, ncrtc %d != 1",
-                   x11->randr.outputs[i]->ncrtc);
-            return 0;
-        }
-        crtc = crtc_from_id(x11, x11->randr.outputs[i]->crtcs[0]);
-        if (!crtc) {
-            syslog(LOG_ERR, "error: inconsistent or stale data from X");
-            return 0;
-        }
-        client_mode = &mon->monitors[i];
-        mode = mode_from_id(x11, crtc->mode);
-        if (!mode) {
-            return 0;
-        }
+    for (i = 0; i < conf1->num_of_monitors; i++) {
+        VDAgentMonConfig *mon1 = &conf1->monitors[i];
+        VDAgentMonConfig *mon2 = &conf2->monitors[i];
         /* NOTE: we don't compare depth. */
-        /* NOTE 2: width set by X is a multiple of 8, so ignore lower 3 bits */
-        if ((mode->width & ~7) != (client_mode->width & ~7) ||
-            mode->height != client_mode->height ||
-            (crtc->x & ~7) != (client_mode->x & ~7) ||
-            crtc->y != client_mode->y) {
+        if (mon1->x != mon2->x || mon1->y != mon2->y ||
+               mon1->width != mon2->width || mon1->height != mon2->height)
             return 0;
-        }
     }
     return 1;
 }
 
+static int config_size(int num_of_monitors)
+{
+    return sizeof(VDAgentMonitorsConfig) +
+                           num_of_monitors * sizeof(VDAgentMonConfig);
+}
+
+static VDAgentMonitorsConfig *get_current_mon_config(struct vdagent_x11 *x11)
+{
+    int i, num_of_monitors = 0;
+    XRRModeInfo *mode;
+    XRRCrtcInfo *crtc;
+    XRRScreenResources *res = x11->randr.res;
+    VDAgentMonitorsConfig *mon_config;
+
+    mon_config = calloc(1, config_size(res->noutput));
+    if (!mon_config) {
+        syslog(LOG_ERR, "out of memory allocating current monitor config");
+        return NULL;
+    }
+
+    for (i = 0 ; i < res->noutput; i++) {
+        if (x11->randr.outputs[i]->ncrtc == 0)
+            continue; /* Monitor disabled, already zero-ed by calloc */
+        if (x11->randr.outputs[i]->ncrtc != 1)
+            goto error;
+
+        crtc = crtc_from_id(x11, x11->randr.outputs[i]->crtcs[0]);
+        if (!crtc)
+            goto error;
+
+        mode = mode_from_id(x11, crtc->mode);
+        if (!mode)
+            continue; /* Monitor disabled, already zero-ed by calloc */
+
+        mon_config->monitors[i].x      = crtc->x;
+        mon_config->monitors[i].y      = crtc->y;
+        mon_config->monitors[i].width  = mode->width;
+        mon_config->monitors[i].height = mode->height;
+        num_of_monitors = i + 1;
+    }
+    mon_config->num_of_monitors = num_of_monitors;
+    mon_config->flags = VD_AGENT_CONFIG_MONITORS_FLAG_USE_POS;
+    return mon_config;
+
+error:
+    syslog(LOG_ERR, "error: inconsistent or stale data from X");
+    free(mon_config);
+    return NULL;
+}
+
 static void dump_monitors_config(struct vdagent_x11 *x11,
                                  VDAgentMonitorsConfig *mon_config,
                                  const char *prefix)
@@ -669,6 +667,8 @@ static void dump_monitors_config(struct vdagent_x11 *x11,
            mon_config->flags);
     for (i = 0 ; i < mon_config->num_of_monitors; ++i) {
         m = &mon_config->monitors[i];
+        if (!monitor_enabled(m))
+            continue;
         syslog(LOG_DEBUG, "received monitor %d config %dx%d+%d+%d", i,
                m->width, m->height, m->x, m->y);
     }
@@ -685,19 +685,20 @@ static void dump_monitors_config(struct vdagent_x11 *x11,
  *  invalid configuration request from client.
  */
 void vdagent_x11_set_monitor_config(struct vdagent_x11 *x11,
-                                    VDAgentMonitorsConfig *mon_config)
+                                    VDAgentMonitorsConfig *mon_config,
+                                    int fallback)
 {
-    int i;
     int width, height;
     int x, y;
     int primary_w, primary_h;
+    int i, real_num_of_monitors = 0;
+    VDAgentMonitorsConfig *curr = NULL;
 
     if (!x11->has_xrandr)
         goto exit;
 
-    if (mon_config->num_of_monitors < 1) {
-        syslog(LOG_ERR, "client sent invalid monitor config number %d",
-               mon_config->num_of_monitors);
+    if (enabled_monitors(mon_config) < 1) {
+        syslog(LOG_ERR, "client sent config with all monitors disabled");
         goto exit;
     }
 
@@ -705,6 +706,20 @@ void vdagent_x11_set_monitor_config(struct vdagent_x11 *x11,
         dump_monitors_config(x11, mon_config, "from guest");
     }
 
+    for (i = 0; i < mon_config->num_of_monitors; i++) {
+        if (monitor_enabled(&mon_config->monitors[i]))
+            real_num_of_monitors = i + 1;
+    }
+    mon_config->num_of_monitors = real_num_of_monitors;
+
+    update_randr_res(x11, 0);
+    if (mon_config->num_of_monitors > x11->randr.res->noutput) {
+        syslog(LOG_WARNING,
+               "warning unexpected client request: #mon %d > driver output %d",
+               mon_config->num_of_monitors, x11->randr.res->noutput);
+        mon_config->num_of_monitors = x11->randr.res->noutput;
+    }
+
     if (mon_config->num_of_monitors > MONITOR_SIZE_COUNT) {
         syslog(LOG_WARNING, "warning: client send %d monitors, capping at %d",
                mon_config->num_of_monitors, MONITOR_SIZE_COUNT);
@@ -715,50 +730,74 @@ void vdagent_x11_set_monitor_config(struct vdagent_x11 *x11,
 
     constrain_to_screen(x11, &primary_w, &primary_h);
 
-    if (same_monitor_configs(x11, mon_config, primary_w, primary_h)) {
+    if (x11->debug) {
+        dump_monitors_config(x11, mon_config, "after zeroing");
+    }
+
+    curr = get_current_mon_config(x11);
+    if (same_monitor_configs(mon_config, curr) &&
+           x11->width[0] == primary_w && x11->height[0] == primary_h) {
         goto exit;
     }
 
-    if (x11->debug) {
-        dump_monitors_config(x11, mon_config, "after zeroing");
+    if (same_monitor_configs(mon_config, x11->randr.failed_conf)) {
+        syslog(LOG_WARNING, "Ignoring previous failed client monitor config");
+        goto exit;
     }
 
     for (i = mon_config->num_of_monitors; i < x11->randr.res->noutput; i++)
         xrandr_disable_output(x11, i);
 
     for (i = 0; i < mon_config->num_of_monitors; ++i) {
+        if (!monitor_enabled(&mon_config->monitors[i])) {
+            xrandr_disable_output(x11, i);
+            continue;
+        }
         /* Try to create the requested resolution */
         width = mon_config->monitors[i].width;
         height = mon_config->monitors[i].height;
         x = mon_config->monitors[i].x;
         y = mon_config->monitors[i].y;
         if (!xrandr_add_and_set(x11, i, x, y, width, height) &&
-                mon_config->num_of_monitors == 1) {
+                enabled_monitors(mon_config) == 1) {
             set_screen_to_best_size(x11, width, height,
                                     &primary_w, &primary_h);
             goto update;
         }
     }
 
-    if (primary_w != x11->width || primary_h != x11->height) {
+    if (primary_w != x11->width[0] || primary_h != x11->height[0]) {
         if (x11->debug)
             syslog(LOG_DEBUG, "Changing screen size to %dx%d",
                    primary_w, primary_h);
-        arm_error_handler(x11);
-        XRRSetScreenSize(x11->display, x11->root_window, primary_w, primary_h,
-                         DisplayWidthMM(x11->display, x11->screen),
-                         DisplayHeightMM(x11->display, x11->screen));
-        if (check_error_handler(x11)) {
-            syslog(LOG_ERR, "failed to XRRSetScreenSize");
-            x11->set_crtc_config_not_functional = 1;
+        vdagent_x11_set_error_handler(x11, error_handler);
+        XRRSetScreenSize(x11->display, x11->root_window[0], primary_w, primary_h,
+                         DisplayWidthMM(x11->display, 0),
+                         DisplayHeightMM(x11->display, 0));
+        if (vdagent_x11_restore_error_handler(x11)) {
+            syslog(LOG_ERR, "XRRSetScreenSize failed, not enough mem?");
+            if (!fallback && curr) {
+                syslog(LOG_WARNING, "Restoring previous config");
+                vdagent_x11_set_monitor_config(x11, curr, 1);
+                free(curr);
+                /* Remember this config failed, if the client is maximized or
+                   fullscreen it will keep sending the failing config. */
+                free(x11->randr.failed_conf);
+                x11->randr.failed_conf =
+                    malloc(config_size(mon_config->num_of_monitors));
+                if (x11->randr.failed_conf)
+                    memcpy(x11->randr.failed_conf, mon_config,
+                           config_size(mon_config->num_of_monitors));
+                return;
+            }
         }
     }
 
 update:
     update_randr_res(x11,
-        x11->randr.num_monitors != mon_config->num_of_monitors);
-    x11->width = primary_w;
-    x11->height = primary_h;
+        x11->randr.num_monitors != enabled_monitors(mon_config));
+    x11->width[0] = primary_w;
+    x11->height[0] = primary_h;
 
     /* Flush output buffers and consume any pending events (ConfigureNotify) */
     x11->dont_send_guest_xorg_res = 1;
@@ -766,36 +805,55 @@ update:
     x11->dont_send_guest_xorg_res = 0;
 
 exit:
-    vdagent_x11_send_daemon_guest_xorg_res(x11);
+    vdagent_x11_send_daemon_guest_xorg_res(x11, 0);
 
     /* Flush output buffers and consume any pending events */
     vdagent_x11_do_read(x11);
+    free(curr);
 }
 
-void vdagent_x11_send_daemon_guest_xorg_res(struct vdagent_x11 *x11)
+void vdagent_x11_send_daemon_guest_xorg_res(struct vdagent_x11 *x11, int update)
 {
     struct vdagentd_guest_xorg_resolution *res = NULL;
-    XineramaScreenInfo *screen_info = NULL;
-    int i, screen_count = 0;
+    int i, width = 0, height = 0, screen_count = 0;
+
+    if (x11->has_xrandr) {
+        VDAgentMonitorsConfig *curr;
+
+        if (update)
+            update_randr_res(x11, 0);
+
+        curr = get_current_mon_config(x11);
+        if (!curr)
+            goto no_info;
+
+        screen_count = curr->num_of_monitors;
+        res = malloc(screen_count * sizeof(*res));
+        if (!res) {
+            free(curr);
+            goto no_mem;
+        }
+
+        for (i = 0; i < screen_count; i++) {
+            res[i].width  = curr->monitors[i].width;
+            res[i].height = curr->monitors[i].height;
+            res[i].x = curr->monitors[i].x;
+            res[i].y = curr->monitors[i].y;
+        }
+        free(curr);
+        width  = x11->width[0];
+        height = x11->height[0];
+    } else if (x11->has_xinerama) {
+        XineramaScreenInfo *screen_info = NULL;
 
-    if (x11->has_xinerama) {
-        /* Xinerama reports the same information RANDR reports, so stay
-         * with Xinerama for support of Xinerama only setups */
         screen_info = XineramaQueryScreens(x11->display, &screen_count);
-    }
-
-    if (screen_count == 0)
-        screen_count = 1;
-
-    res = malloc(screen_count * sizeof(*res));
-    if (!res) {
-        syslog(LOG_ERR, "out of memory while trying to send resolutions, not sending resolutions.");
-        if (screen_info)
+        if (!screen_info)
+            goto no_info;
+        res = malloc(screen_count * sizeof(*res));
+        if (!res) {
             XFree(screen_info);
-        return;
-    }
-
-    if (screen_info) {
+            goto no_mem;
+        }
         for (i = 0; i < screen_count; i++) {
             if (screen_info[i].screen_number >= screen_count) {
                 syslog(LOG_ERR, "Invalid screen number in xinerama screen info (%d >= %d)",
@@ -810,14 +868,36 @@ void vdagent_x11_send_daemon_guest_xorg_res(struct vdagent_x11 *x11)
             res[screen_info[i].screen_number].y = screen_info[i].y_org;
         }
         XFree(screen_info);
+        width  = x11->width[0];
+        height = x11->height[0];
     } else {
-        res[0].width  = x11->width;
-        res[0].height = x11->height;
-        res[0].x = 0;
-        res[0].y = 0;
+no_info:
+        screen_count = x11->screen_count;
+        res = malloc(screen_count * sizeof(*res));
+        if (!res)
+            goto no_mem;
+        for (i = 0; i < screen_count; i++) {
+            res[i].width  = x11->width[i];
+            res[i].height = x11->height[i];
+            /* No way to get screen coordinates, assume rtl order */
+            res[i].x = width;
+            res[i].y = 0;
+            width += x11->width[i];
+            if (x11->height[i] > height)
+                height = x11->height[i];
+        }
     }
 
-    udscs_write(x11->vdagentd, VDAGENTD_GUEST_XORG_RESOLUTION, x11->width,
-                x11->height, (uint8_t *)res, screen_count * sizeof(*res));
+    if (x11->debug) {
+        for (i = 0; i < screen_count; i++)
+            syslog(LOG_DEBUG, "Screen %d %dx%d%+d%+d", i, res[i].width,
+                   res[i].height, res[i].x, res[i].y);
+    }
+
+    udscs_write(x11->vdagentd, VDAGENTD_GUEST_XORG_RESOLUTION, width, height,
+                (uint8_t *)res, screen_count * sizeof(*res));
     free(res);
+    return;
+no_mem:
+    syslog(LOG_ERR, "out of memory while trying to send resolutions, not sending resolutions.");
 }
