@@ -43,6 +43,7 @@
 #include "vdagent-file-xfers.h"
 
 static const char *portdev = "/dev/virtio-ports/com.redhat.spice.0";
+static const char *vdagentd_socket = VDAGENTD_SOCKET;
 static int debug = 0;
 static const char *fx_dir = NULL;
 static int fx_open_dir = -1;
@@ -103,6 +104,7 @@ void daemon_read_complete(struct udscs_connection **connp,
         free(data);
         break;
     case VDAGENTD_CLIENT_DISCONNECTED:
+        vdagent_x11_client_disconnected(x11);
         vdagent_file_xfers_destroy(vdagent_file_xfers);
         vdagent_file_xfers = vdagent_file_xfers_create(client, fx_dir,
                                                        fx_open_dir, debug);
@@ -117,7 +119,7 @@ void daemon_read_complete(struct udscs_connection **connp,
 int client_setup(int reconnect)
 {
     while (!quit) {
-        client = udscs_connect(VDAGENTD_SOCKET, daemon_read_complete, NULL,
+        client = udscs_connect(vdagentd_socket, daemon_read_complete, NULL,
                                vdagentd_messages, VDAGENTD_NO_MESSAGES,
                                debug);
         if (client || !reconnect || quit) {
@@ -132,15 +134,16 @@ static void usage(FILE *fp)
 {
     fprintf(fp,
       "Usage: spice-vdagent [OPTIONS]\n\n"
-      "Spice guest agent X11 session agent.\n\n"
+      "Spice guest agent X11 session agent, version %s.\n\n"
       "Options:\n"
       "  -h                                print this text\n"
       "  -d                                log debug messages\n"
       "  -s <port>                         set virtio serial port\n"
+      "  -S <filename>                     set udcs socket\n"
       "  -x                                don't daemonize\n"
       "  -f <dir|xdg-desktop|xdg-download> file xfer save dir\n"
-      "  -o <0|1>                          open dir on file xfer completion\n"
-      );
+      "  -o <0|1>                          open dir on file xfer completion\n",
+      VERSION);
 }
 
 static void quit_handler(int sig)
@@ -183,7 +186,7 @@ int main(int argc, char *argv[])
     struct sigaction act;
 
     for (;;) {
-        if (-1 == (c = getopt(argc, argv, "-dxhys:f:o:")))
+        if (-1 == (c = getopt(argc, argv, "-dxhys:f:o:S:")))
             break;
         switch (c) {
         case 'd':
@@ -207,6 +210,9 @@ int main(int argc, char *argv[])
         case 'o':
             fx_open_dir = atoi(optarg);
             break;
+        case 'S':
+            vdagentd_socket = optarg;
+            break;
         default:
             fputs("\n", stderr);
             usage(stderr);
@@ -226,9 +232,7 @@ int main(int argc, char *argv[])
             LOG_USER);
 
     if (file_test(portdev) != 0) {
-        syslog(LOG_ERR, "Missing virtio device '%s': %s",
-                portdev, strerror(errno));
-        return 1;
+        return 0;
     }
 
     if (do_daemonize)
