@@ -55,8 +55,10 @@ struct agent_data {
 /* variables */
 static const char *pidfilename = "/var/run/spice-vdagentd/spice-vdagentd.pid";
 static const char *portdev = "/dev/virtio-ports/com.redhat.spice.0";
+static const char *vdagentd_socket = VDAGENTD_SOCKET;
 static const char *uinput_device = "/dev/uinput";
 static int debug = 0;
+static int uinput_fake = 0;
 static struct udscs_server *server = NULL;
 static struct vdagent_virtio_port *virtio_port = NULL;
 static GHashTable *active_xfers = NULL;
@@ -95,6 +97,7 @@ static void send_capabilities(struct vdagent_virtio_port *vport,
     VD_AGENT_SET_CAPABILITY(caps->caps, VD_AGENT_CAP_CLIPBOARD_BY_DEMAND);
     VD_AGENT_SET_CAPABILITY(caps->caps, VD_AGENT_CAP_CLIPBOARD_SELECTION);
     VD_AGENT_SET_CAPABILITY(caps->caps, VD_AGENT_CAP_SPARSE_MONITORS_CONFIG);
+    VD_AGENT_SET_CAPABILITY(caps->caps, VD_AGENT_CAP_GUEST_LINEEND_LF);
 
     vdagent_virtio_port_write(vport, VDP_CLIENT_PORT,
                               VD_AGENT_ANNOUNCE_CAPABILITIES, 0,
@@ -185,7 +188,7 @@ static void do_client_clipboard(struct vdagent_virtio_port *vport,
 
     if (!active_session_conn) {
         syslog(LOG_WARNING,
-               "Could not find an agent connnection belonging to the "
+               "Could not find an agent connection belonging to the "
                "active session, ignoring client clipboard request");
         return;
     }
@@ -315,7 +318,8 @@ int virtio_port_read_complete(
                                                 agent_data->height,
                                                 agent_data->screen_info,
                                                 agent_data->screen_count,
-                                                debug > 1);
+                                                debug > 1,
+                                                uinput_fake);
             if (!uinput) {
                 syslog(LOG_CRIT, "Fatal uinput error");
                 retval = 1;
@@ -486,7 +490,8 @@ static void check_xorg_resolution(void)
                                             agent_data->height,
                                             agent_data->screen_info,
                                             agent_data->screen_count,
-                                            debug > 1);
+                                            debug > 1,
+                                            uinput_fake);
         else
             vdagentd_uinput_update_size(&uinput,
                                         agent_data->width,
@@ -718,20 +723,21 @@ static void usage(FILE *fp)
 {
     fprintf(fp,
             "Usage: spice-vdagentd [OPTIONS]\n\n"
-            "Spice guest agent daemon.\n\n"
+            "Spice guest agent daemon, version %s.\n\n"
             "Options:\n"
-            "  -h         print this text\n"
-            "  -d         log debug messages (use twice for extra info)\n"
-            "  -s <port>  set virtio serial port  [%s]\n"
-            "  -u <dev>   set uinput device       [%s]\n"
-            "  -x         don't daemonize\n"
+            "  -h             print this text\n"
+            "  -d             log debug messages (use twice for extra info)\n"
+            "  -s <port>      set virtio serial port  [%s]\n"
+            "  -S <filename>  set udcs socket [%s]\n"
+            "  -u <dev>       set uinput device       [%s]\n"
+            "  -x             don't daemonize\n"
 #ifdef HAVE_CONSOLE_KIT
-            "  -X         Disable console kit integration\n"
+            "  -X             Disable console kit integration\n"
 #endif
 #ifdef HAVE_LIBSYSTEMD_LOGIN
             "  -X         Disable systemd-logind integration\n"
 #endif
-            ,portdev, uinput_device);
+            ,VERSION, portdev, vdagentd_socket, uinput_device);
 }
 
 void daemonize(void)
@@ -833,7 +839,7 @@ int main(int argc, char *argv[])
     struct sigaction act;
 
     for (;;) {
-        if (-1 == (c = getopt(argc, argv, "-dhxXs:u:")))
+        if (-1 == (c = getopt(argc, argv, "-dhxXs:u:S:")))
             break;
         switch (c) {
         case 'd':
@@ -842,6 +848,9 @@ int main(int argc, char *argv[])
         case 's':
             portdev = optarg;
             break;
+        case 'S':
+            vdagentd_socket = optarg;
+            break;
         case 'u':
             uinput_device = optarg;
             break;
@@ -861,6 +870,11 @@ int main(int argc, char *argv[])
         }
     }
 
+    if (strncmp(uinput_device, "/dev", 4) != 0) {
+        syslog(LOG_INFO, "using fake uinput");
+        uinput_fake = 1;
+    }
+
     memset(&act, 0, sizeof(act));
     act.sa_flags = SA_RESTART;
     act.sa_handler = quit_handler;
@@ -872,18 +886,18 @@ int main(int argc, char *argv[])
     openlog("spice-vdagentd", do_daemonize ? 0 : LOG_PERROR, LOG_USER);
 
     /* Setup communication with vdagent process(es) */
-    server = udscs_create_server(VDAGENTD_SOCKET, agent_connect,
+    server = udscs_create_server(vdagentd_socket, agent_connect,
                                  agent_read_complete, agent_disconnect,
                                  vdagentd_messages, VDAGENTD_NO_MESSAGES,
                                  debug);
     if (!server) {
         syslog(LOG_CRIT, "Fatal could not create server socket %s",
-               VDAGENTD_SOCKET);
+               vdagentd_socket);
         return 1;
     }
-    if (chmod(VDAGENTD_SOCKET, 0666)) {
+    if (chmod(vdagentd_socket, 0666)) {
         syslog(LOG_CRIT, "Fatal could not change permissions on %s: %m",
-               VDAGENTD_SOCKET);
+               vdagentd_socket);
         udscs_destroy_server(server);
         return 1;
     }
@@ -893,7 +907,7 @@ int main(int argc, char *argv[])
 
 #ifdef WITH_STATIC_UINPUT
     uinput = vdagentd_uinput_create(uinput_device, 1024, 768, NULL, 0,
-                                    debug > 1);
+                                    debug > 1, uinput_fake);
     if (!uinput) {
         udscs_destroy_server(server);
         return 1;
@@ -915,8 +929,8 @@ int main(int argc, char *argv[])
     vdagent_virtio_port_destroy(&virtio_port);
     session_info_destroy(session_info);
     udscs_destroy_server(server);
-    if (unlink(VDAGENTD_SOCKET) != 0)
-        syslog(LOG_ERR, "unlink %s: %s", VDAGENTD_SOCKET, strerror(errno));
+    if (unlink(vdagentd_socket) != 0)
+        syslog(LOG_ERR, "unlink %s: %s", vdagentd_socket, strerror(errno));
     syslog(LOG_INFO, "vdagentd quiting, returning status %d", retval);
 
     if (do_daemonize)
